import 'dart:async';

import 'package:get/get.dart';

mixin CountableController on GetxController {
  List<StreamSubscription> _streams = [];
  RxSet<String> _tasks = RxSet();
  RxBool isProcessing = false.obs;

  @override
  void onInit() {
    super.onInit();
    autoDispose(
      _tasks.listen(
        (data) {
          if (isProcessing.value != data.isNotEmpty) {
            isProcessing.toggle();
          }
        },
      ),
    );
  }

  @override
  void dispose() {
    _streams
      ..forEach((element) => element.cancel())
      ..clear();
    super.dispose();
  }

  void add(
    String uuid, {
    Duration? autoComplete,
  }) {
    _tasks.add(uuid);
    if (autoComplete != null) {
      autoDispose(
        Stream.fromFuture(
          Future.delayed(autoComplete),
        ).listen(
          (_) => complete(uuid),
        ),
      );
    }
  }

  void complete(String uuid) => _tasks.remove(uuid);

  void autoDispose(StreamSubscription subscription) =>
      _streams.add(subscription);
}
