import 'dart:math';

extension ListX<T> on List<T> {
  T? itemOrNull(int at) {
    if (isNotEmpty && 0 <= at && at < length) {
      return this[at];
    }
    return null;
  }

  T? get firstOrNull => itemOrNull(0);

  T? get lastOrNull => itemOrNull(length - 1);

  List<E> mapIndexed<E>(
    E Function(int index, T model) mapper,
  ) {
    return asMap()
        .map((key, value) => MapEntry(key, mapper(key, value)))
        .values
        .toList();
  }

  List<T> dropFirst([int first = 1]) => sublist(0, first);

  List<T> dropLast([int last = 1]) => sublist(0, max(length, length - last));
}

extension FlattableList<T> on Iterable<Iterable<T>> {
  Iterable<E> flatMap<E>(E Function(T item) mapper) {
    return expand((element) => element).map(mapper);
  }
}

extension NullableListX<T> on Iterable<T?> {
  Iterable<T> filterNull() {
    return where((element) => element != null).map((e) => e as T);
  }
}
